/*  Copyright (C) 2009, Jean-Baptiste LANGLOIS

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2
    as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

// The ParseObject, ParseString and ParseValue methods are based on Patrick van Bergen's

using System;
using System.Collections;
using System.Text;

namespace System.Json
{
    /// <summary>
    /// Parse the JSON data to deserialize it as JsonData
    /// </summary>
    /// <seealso cref="JsonData"/>
    public class JsonAttributes
    {

        /// <summary>
        /// Constructor of the JsonAttributes class
        /// </summary>
        /// <example>JsonAttributes json = new JsonAttributes();</example>
        public JsonAttributes()
        {

        }

        /// <summary>
        /// Parse the current position of the JSON data to get the next value.
        /// </summary>
        /// <param name="data">The JSON data as a character array</param>
        /// <param name="pos">Current position in the JSON data</param>
        /// <returns>an Object (mostly JsonData) referring to the value</returns>
        /// <remarks>The current position variable must be a pointer</remarks>
        /// <seealso cref="JsonData"/>
        /// <exception cref="JsonFormatException">Can happen if some brackets are missing</exception>
        public object ParseValue(char[] data, ref int pos)
        {
            switch (GetType(data, pos))
            {
                case JsonType.String:
                    return ParseString(data, ref pos);
                case JsonType.Object:
                    return ParseObject(data, ref pos);
                case JsonType.Number:
                    return ParseNumber(data, ref pos);
                case JsonType.Array:
                    return ParseArray(data, ref pos);
                case JsonType.True:
                    NextChar(data, ref pos);
                    return true;
                case JsonType.False:
                    NextChar(data, ref pos);
                    return false;
                case JsonType.Null:
                    NextChar(data, ref pos);
                    return null;
                case JsonType.None:
                    return null;
            }
            return null;
        }

        private string ParseString(char[] data, ref int pos)
        {
            string s = "";
            char c;

            pos = TrimLeft(data, pos);

            //c = data[pos++];
            pos++; // Suppress the "

            bool complete = false;
            while (!complete)
            {
                if (pos == data.Length)
                {
                    break;
                }
                c = data[pos++];

                if (c == '"')
                {
                    complete = true;
                    break;
                }
                else if (c == '\\')
                {
                    if (pos == data.Length)
                    {
                        break;
                    }
                    switch (data[pos++])
                    {
                        case '"':
                            s += '"';
                            break;
                        case '\\':
                            s += '\\';
                            break;
                        case '/':
                            s += '/';
                            break;
                        case 'b':
                            s += '\b';
                            break;
                        case 'f':
                            s += '\f';
                            break;
                        case 'n':
                            s += '\n';
                            break;
                        case 'r':
                            s += '\r';
                            break;
                        case 't':
                            s += '\t';
                            break;
                    }
                }
                else
                {
                    s += c;
                }
            }

            if (!complete)
            {
                throw new JsonFormatException("Missing quotes in character " + pos.ToString(), pos);
                //return null;
            }

            return s;
        }

        private double ParseNumber(char[] data, ref int pos)
        {
            pos = TrimLeft(data, pos);
            int lastIndex = pos;
            for (lastIndex = pos; (lastIndex < data.Length) && ("0123456789+-.eE".IndexOf(data[lastIndex]) > -1); lastIndex++) ;
            lastIndex--;
            char[] numberCharArray = new char[(lastIndex - pos) + 1];

            Array.Copy(data, pos, numberCharArray, 0, (lastIndex - pos) + 1);
            pos = lastIndex + 1;
            return Double.Parse(new string(numberCharArray));
        }

        private object[] ParseArray(char[] data, ref int pos)
        {
            ArrayList array = new ArrayList();
            NextChar(data, ref pos);
            while (true)
            {
                if (GetType(data, pos) == JsonType.None)
                {
                    if (data.Length == pos)
                        throw new JsonFormatException("Missing squared bracket in character " + pos.ToString(), pos);
                    return null;
                }
                else if (GetType(data, pos) == JsonType.Comma)
                {
                    NextChar(data, ref pos);
                }
                else if (GetType(data, pos) == JsonType.ArrayClose)
                {
                    NextChar(data, ref pos);
                    break;
                }
                else
                {
                    object value = ParseValue(data, ref pos);
                    array.Add(value);
                }
            }

            return (object[])array.ToArray(typeof(object));
        }

        private JsonData[] ParseObject(char[] data, ref int pos)
        {
            ArrayList arl = new ArrayList();
            //int activeChar = 0;
            NextChar(data, ref pos);
            while (true)
            {
                switch (GetType(data, pos))
                {
                    case JsonType.None:
                        if (data.Length == pos)
                            throw new JsonFormatException("Missing curly bracket in character " + pos.ToString(), pos);
                        return null;
                    case JsonType.Comma:
                        NextChar(data, ref pos);
                        break;
                    case JsonType.ObjectClose:
                        NextChar(data, ref pos);
                        return (JsonData[])arl.ToArray(typeof(JsonData));
                    default:
                        JsonData node = new JsonData();
                        string name = ParseString(data, ref pos);
                        if (name == null)
                        {
                            throw new JsonFormatException("Invalid string variable on character " + pos.ToString(), pos);
                            //return null;
                        }
                        node.Name = name;

                        if (NextChar(data, ref pos) != JsonType.Colon)
                        {
                            throw new JsonFormatException("Colon expected on character " + pos.ToString(), pos);
                            //return null;
                        }

                        node.Type = GetType(data, pos);
                        object value = ParseValue(data, ref pos);
                        if (value == null)
                        {
                            throw new JsonFormatException("Invalid variable on character " + pos.ToString(), pos);
                            //return null;
                        }
                        node.Value = value;
                        arl.Add(node);
                        break;
                }
            }
            //return (JsonData[])arl.ToArray(typeof(JsonData));
        }

        private JsonType NextChar(char[] data, ref int pos)
        {
            pos = TrimLeft(data, pos);
            if (pos == data.Length)
                return JsonType.None;
            switch (data[pos++])
            {
                case '{':
                    return JsonType.Object;
                case '}':
                    return JsonType.ObjectClose;
                case '[':
                    return JsonType.Array;
                case ']':
                    return JsonType.ArrayClose;
                case ',':
                    return JsonType.Comma;
                case ':':
                    return JsonType.Colon;
                case '"':
                    return JsonType.String;
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                case '-':
                    return JsonType.Number;
                default:
                    if (pos + 4 < data.Length)
                    {
                        if (
                            (data[pos - 1] == 'f') &
                            (data[pos] == 'a') &
                            (data[pos + 1] == 'l') &
                            (data[pos + 2] == 's') &
                            (data[pos + 3] == 'e'))
                        {
                            pos += 4;
                            return JsonType.False;
                        }
                    }
                    if (pos + 3 < data.Length)
                    {
                        if (
                            (data[pos - 1] == 't') &
                            (data[pos] == 'r') &
                            (data[pos + 1] == 'u') &
                            (data[pos + 2] == 'e'))
                        {
                            pos += 3;
                            return JsonType.True;
                        }
                        if (
                            (data[pos - 1] == 'n') &
                            (data[pos] == 'u') &
                            (data[pos + 1] == 'l') &
                            (data[pos + 2] == 'l'))
                        {
                            pos += 3;
                            return JsonType.Null;
                        }
                    }
                    return JsonType.None;
            }
        }

        /// <summary>
        /// Parse the current position of the JSON data to determine which type of data is coming next.
        /// </summary>
        /// <param name="data">The JSON data as a character array</param>
        /// <param name="pos">Current position in the JSON data</param>
        /// <returns>Type of the coming data</returns>
        /// <seealso cref="JsonType"/>
        public JsonType GetType(char[] data, int pos)
        {
            int saveIndex = pos;
            return NextChar(data, ref saveIndex);
        }

        private int TrimLeft(char[] data, int pos)
        {
            while ((pos < data.Length) &&
                ((data[pos] == ' ') ||
                (data[pos] == '\t') ||
                (data[pos] == '\r') ||
                (data[pos] == '\n')))
            {
                if (pos == data.Length)
                    break;
                pos++;
            }
            return pos;
        }
    }
}