/*  Copyright (C) 2009, Jean-Baptiste LANGLOIS

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2
    as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

using System;
using System.Collections;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace System.Json
{
    /// <summary>
    /// Manage the JSON file to deserialize
    /// </summary>
    /// <example>
    /// <code>
    /// JsonDocument js = new JsonDocument();
    /// js.Load("test.json");
    /// foreach (JsonData node in js.Attributes)
    /// {
    ///     Console.WriteLine("Name : " + node.Name + ", Type : " + node.Type);
    /// }
    /// </code>
    /// </example>
    /// <seealso cref="JsonAttributes"/>
    public class JsonDocument
    {
        private JsonData[] json_data = null;

        /// <summary>
        /// Constructor of the JsonDocument class
        /// </summary>
        public JsonDocument()
        {

        }

        /// <summary>
        /// Load the file in memory, check if it correct and deserialize it
        /// </summary>
        /// <param name="file">fileName of the JSON file</param>
        /// <returns>'true' if the file was correct, 'false' otherwise</returns>
        public bool Load(string file)
        {
            StreamReader sr = new StreamReader(file);
            string line = sr.ReadToEnd();
            line = Regex.Replace(line, @"\/\/.+\r\n", "\r\n");
            line = Regex.Replace(line, @"/\*(?s:.*?)\*/", "");
            JsonAttributes attr = new JsonAttributes();
            int pos = 0;
            try
            {
                json_data = (JsonData[])attr.ParseValue(line.ToCharArray(), ref pos);
                return true;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Array of JsonData which contains all the deserialized JSON file
        /// </summary>
        /// <seealso cref="JsonData"/>
        public JsonData[] Attributes
        {
            get
            {
                return json_data;
            }
        }
        /// <summary>
        /// Create a XML pattern from the JSON document
        /// </summary>
        /// <returns>a XML character string if a file has been loaded, 'null' otherwise</returns>
        public string ExportXml()
        {
            string result = "";
            if (json_data == null)
            {
                return null;
            }
            foreach (JsonData jsdata in json_data)
            {
                result += ExportValue(jsdata, 0);
            }
            return result;
        }

        private string ExportArray(object jsobj, int pos)
        {
            string result = "";
            string pad = "";
            for (int i = 0; i < pos; i++)
                pad += "  ";
            foreach (object jsdatas in (object[])((JsonData)jsobj).Value)
            {
                string l_result = "";
                if (jsdatas is JsonData[])
                {
                    foreach (JsonData jsdata in (JsonData[])jsdatas)
                    {
                        l_result += ExportValue(jsdata, pos + 1);
                    }
                    result += pad + "<" + ((JsonData)jsobj).Name + ">\r\n" + l_result + pad + "</" + ((JsonData)jsobj).Name + ">\r\n";
                }
                else
                {
                    result += pad + "<" + ((JsonData)jsobj).Name + ">" + jsdatas.ToString() + "</" + ((JsonData)jsobj).Name + ">\r\n";
                }
            }
            return result;
        }

        private string ExportObject(object jsdatas, int pos)
        {
            string result = "";
            string pad = "";
            for (int i = 0; i < pos; i++)
                pad += "  ";
            foreach (JsonData jsdata in (object[])((JsonData)jsdatas).Value)
            {
                result += ExportValue(jsdata, pos + 1);
            }
            return pad + "<" + ((JsonData)jsdatas).Name + ">\r\n" + result + pad + "</" + ((JsonData)jsdatas).Name + ">\r\n";
        }

        private string ExportString(JsonData jdata, int pos)
        {
            string pad = "";
            for (int i = 0; i < pos; i++)
                pad += "  ";
            return pad + "<" + jdata.Name + ">" + jdata.Value.ToString() + "</" + jdata.Name + ">\r\n";
        }

        private string ExportValue(JsonData jdata, int pos)
        {
            switch (jdata.Type)
            {
                case JsonType.Array:
                    return ExportArray(jdata, pos);
                case JsonType.Object:
                    return ExportObject(jdata, pos);
                default:
                    return ExportString(jdata, pos);
            }
        }
    }
}
